package foh.rpg.area.factory.randomizer;

import java.util.LinkedList;
import java.util.List;

import foh.rpg.area.Area;
import foh.rpg.area.Cell;
import foh.rpg.area.factory.AreaPrototype;
import foh.rpg.character.factory.ActorFactory;
import foh.rpg.character.position.Position;
import foh.rpg.util.Dice;

public class RandomAreaPrototype extends AreaPrototype {
	
	private int segmentsX;
	private int segmentsY;
	
	private Cell floorPrototype;
	private Cell wallPrototype;
	
	private List<ActorSpawn> spawns = new LinkedList<ActorSpawn>();
	
	private List<Segment> prebuiltSegments = new LinkedList<Segment>();
	
	@Override
	public Area buildArea() {
		Segment[][] map = new Segment[segmentsX][segmentsY];
		
		// 1st, we place prebuilt segments
		
		for (Segment segment : this.prebuiltSegments) {
			
			// for each segment, get list of possible positions 
			List<Position> locations = getPossibleSegmentLocations(map, segment);
			
			if (locations.size() > 0) {
				// pick one of possible positions randomly and use it
				Position position = locations.get(new Dice(0, locations.size()-1).roll());
				map[position.getX()][position.getY()] = segment;
			} else {
				// no suitable position for our prebuilt? ohshi..
				// TODO: restart generation, handle it, something
			}
			
		}
		
		// naive segment building, can, and will, produce disconected map.
		
		for (int x = 0; x < map.length; x++) {
			for (int y = 0; y < map[x].length; y++) {
				
				if (map[x][y]== null) {
					
					// get surrounding connectors
					TemplateSegment template = getTemplateFor(x, y, map);
					List<Segment> list = SegmentEnumenator.getInstance().getSegmentsFor(template);
					
					if (list.size() == 0) {
						// uh-oh, we do not have any segment that matches this?? impossible!
						continue;
					} else {
						 map[x][y] = list.get(new Dice(0, list.size()-1).roll());
					}
				
				}
				
			}
		}
		
		// convert to map
		
		Area area = new Area(this.getWallPrototype(), this.getSegmentsX()*9, this.getSegmentsY()*9);
		
		for (int x = 0; x < map.length; x++) {
			for (int y = 0; y < map[x].length; y++) {
				
				map[x][y].render(x*9, y*9, area, this);
				
			}
		}
		
		// populate with monsters
		
		for (ActorSpawn spawn : this.spawns) {
			for (int i = 0; i < spawn.getDensity(); i++) {
				int x = new Dice(0, area.getWidth()-1).roll();
				int y = new Dice(0, area.getHeight()-1).roll();
				
				if (area.getTile()[x][y].isMoveThrough() && area.getTile()[x][y].getActor() == null) {
					area.getTile()[x][y].setActor(ActorFactory.getInstance().getActor(spawn.getActorKey()));
					area.getTile()[x][y].getActor().setSpawnPoint(area.getTile()[x][y].getSpawnPoint());
				} 
			}
		}
		
		//Area2PngRenderer.render(area);
		
		return area;
	}
	
	private List<Position> getPossibleSegmentLocations(Segment[][] map, Segment segment) {
		List<Position> positions = new LinkedList<Position>();
		
		for (int x = 0; x < map.length; x++) {
			for (int y = 0; y < map[x].length; y++) {
				
				if (map[x][y] == null) {
					// empty, check surroundings.
					
					TemplateSegment template = getTemplateFor(x, y, map);
					
					if (template.matches(segment)) {
						positions.add(new Position(x, y));
					}
					
				}
				
			}
		}
		
		return positions;
	}
	
	private TemplateSegment getTemplateFor(int x, int y, Segment[][] map) {
		
		TemplateSegment template = new TemplateSegment();
		
		if (x == 0) {
			template.setWestConnector(SegmentConnector.BLOCKED);
		} else {
			if (map[x-1][y] != null) {
				template.setWestConnector(map[x-1][y].getEastConnector());
			} else {
				template.setWestConnector(null);
			}
		}

		if (x == map.length-1) {
			template.setEastConnector(SegmentConnector.BLOCKED);
		} else {
			if (map[x+1][y] != null) {
				template.setEastConnector(map[x+1][y].getSouthConnector());
			} else {
				template.setEastConnector(null);
			}
		}
		
		if (y == 0) {
			template.setNorthConnector(SegmentConnector.BLOCKED);
		} else {
			if (map[x][y-1] != null) {
				template.setNorthConnector(map[x][y-1].getSouthConnector());
			} else {
				template.setNorthConnector(null);
			}
		}
		
		if (y == map[x].length-1) {
			template.setSouthConnector(SegmentConnector.BLOCKED);
		} else {
			if (map[x][y+1] != null) {
				template.setSouthConnector(map[x][y+1].getSouthConnector());
			} else {
				template.setSouthConnector(null);
			}
		}
		
		return template;
	}

	public void setSegmentsX(int segmentsX) {
		this.segmentsX = segmentsX;
	}

	public int getSegmentsX() {
		return this.segmentsX;
	}

	public void setSegmentsY(int segmentsY) {
		this.segmentsY = segmentsY;
	}

	public int getSegmentsY() {
		return this.segmentsY;
	}

	public void setFloorPrototype(Cell floorPrototype) {
		this.floorPrototype = floorPrototype;
	}

	public Cell getFloorPrototype() {
		return this.floorPrototype;
	}

	public void setWallPrototype(Cell wallPrototype) {
		this.wallPrototype = wallPrototype;
	}

	public Cell getWallPrototype() {
		return this.wallPrototype;
	}

	public void setPrebuiltSegments(List<Segment> prebuiltSegments) {
		this.prebuiltSegments = prebuiltSegments;
	}

	public List<Segment> getPrebuiltSegments() {
		return this.prebuiltSegments;
	}

	public void setSpawns(List<ActorSpawn> spawns) {
		this.spawns = spawns;
	}

	public List<ActorSpawn> getSpawns() {
		return spawns;
	}

}
